/*
 * Support for executing code in new tasks and threads.
 *
 * Copyright (C) 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include <l4/re/env.h>
#include <l4/sys/err.h>
#include <l4/util/util.h>

#include <fsclient/process.h>
#include <systypes/format.h>

#include <stdio.h>



int main(int argc, char *argv[])
{
  long err;

  if (argc < 2)
  {
    printf("Need the program to run.\n");
    return 1;
  }

  /* Create a new process structure. */

  process_t *process = process_new();

  /* Start a process for the given program. */

  err = process_start(process, argc - 1, argv + 1);

  if (err)
  {
    printf("Could not start process: %s\n", l4sys_errtostr(err));
    return 1;
  }

  printf("Finished program initiation.\n");

  /* Obtain the common notifier. */

  process_notifier_t *notifier = process_notify_task();

  /* Subscribe to the process for notifications. */

  err = process_notify_subscribe(process, NOTIFY_TASK_SIGNAL, notifier);

  if (err)
  {
    printf("Could not subscribe to process: %s\n", l4sys_errtostr(err));
    return 1;
  }

  /* Wait for a signal from the process. */

  err = process_notify_wait_process(process, notifier);

  if (err)
  {
    printf("Could not wait for process: %s\n", l4sys_errtostr(err));
    return 1;
  }

  notify_flags_t flags = process_notifications(process);
  notify_values_t values = process_notification_values(process);

  printf("Notified with flags: %" pFMTnotify_flags "x\n", flags);
  printf("Notified with values: %ld, %ld\n", values.sig, values.val);

  printf("End of test.\n");
  return 0;
}

/* vim: tabstop=2 expandtab shiftwidth=2
*/
